import asyncio
from typing import TYPE_CHECKING

from nonebot import on_command
from nonebot.adapters.onebot.v11 import ActionFailed, Message
from nonebot.exception import FinishedException
from nonebot.internal.matcher import Matcher
from nonebot.log import logger
from nonebot.params import CommandArg

from ..data.gamekee import game_kee_calender
from ..data.schaledb import schale_calender

if TYPE_CHECKING:
    from . import HelpList

help_list: "HelpList" = [
    {
        "func": "日程表",
        "trigger_method": "指令",
        "trigger_condition": "ba日程表",
        "brief_des": "查看活动日程表",
        "detail_des": (
            "查看当前未结束的卡池、活动以及起止时间，"
            "默认为GameKee源，可以在指令后带参数使用SchaleDB数据源\n"
            " \n"
            "指令示例：\n"
            "- <ft color=(238,120,0)>ba日程表</ft> （GameKee源）\n"
            "- <ft color=(238,120,0)>ba日程表 schaledb</ft> （SchaleDB源，日服国际服一起发）\n"
            "- <ft color=(238,120,0)>ba日程表 schale 国际服</ft> （SchaleDB源，国际服）"
        ),
    },
]


cmd_calender = on_command("ba日程表")


@cmd_calender.handle()
async def _(matcher: Matcher, cmd_arg: Message = CommandArg()):
    arg: str = cmd_arg.extract_plain_text()

    await matcher.send("正在绘制图片，请稍等")
    try:
        if "s" in (arg := arg.lower()):
            arg = arg.replace("schale", "").strip()
            servers = []

            if (not arg) or ("日" in arg) or ("j" in arg):
                servers.append(0)
            if (not arg) or ("国" in arg) or ("g" in arg):
                servers.append(1)

            await asyncio.gather(
                *[
                    matcher.send(x)
                    for x in (
                        await asyncio.gather(*[schale_calender(x) for x in servers])
                    )
                ],
            )
            await matcher.finish()
        else:
            await matcher.finish(await game_kee_calender())
    except (FinishedException, ActionFailed):  # type: ignore
        raise
    except Exception:
        logger.exception("绘制日程表图片出错")
        await matcher.finish("绘制日程表图片出错，请检查后台输出")



import asyncio
import datetime
from io import BytesIO
from typing import Any, Dict, List, Literal, Optional, TypedDict, cast, overload

from nonebot.adapters.onebot.v11 import MessageSegment
from pil_utils import BuildImage

from ..config import config
from ..util import async_req, recover_alia


class MangaDict(TypedDict):
    cid: int
    title: str
    detail: str
    pics: List[str]


@overload
async def db_get(suffix: str, raw: Literal[False] = False) -> Any:
    ...


@overload
async def db_get(suffix: str, raw: Literal[True] = True) -> bytes:
    ...


async def db_get(suffix: str, raw=False):
    return await async_req(f"{config.ba_bawiki_db_url}{suffix}", raw=raw)  # type: ignore


async def db_get_wiki_data() -> Dict[str, Any]:
    return await db_get("data/wiki.json")


async def db_get_stu_alias() -> Dict[str, List[str]]:
    return await db_get("data/stu_alias.json")


async def db_get_schale_to_gamekee() -> Dict[str, str]:
    return await db_get("data/schale_to_gamekee.json")


async def db_get_extra_l2d_list() -> Dict[str, List[str]]:
    return await db_get("data/extra_l2d_list.json")


async def db_get_raid_alias() -> Dict[str, List[str]]:
    return await db_get("data/raid_alias.json")


async def db_get_terrain_alias() -> Dict[str, List[str]]:
    return await db_get("data/terrain_alias.json")


async def db_get_event_alias() -> Dict[str, List[str]]:
    return await db_get("data/event_alias.json")


async def db_get_gacha_data() -> Dict[str, Any]:
    return await db_get("data/gacha.json")


async def db_get_emoji() -> List[str]:
    return await db_get("data/emoji.json")


async def db_get_manga() -> List[MangaDict]:
    return await db_get("data/manga.json")


async def schale_to_gamekee(o: str) -> str:
    diff = await db_get_schale_to_gamekee()
    if o in diff:
        o = diff[o]
    return o.replace("(", "（").replace(")", "）")


async def recover_stu_alia(a, game_kee=False) -> str:
    ret = recover_alia(a, await db_get_stu_alias())

    if game_kee:
        ret = await schale_to_gamekee(ret)

    return ret


async def db_wiki_stu(name):
    wiki = (await db_get_wiki_data())["student"]
    if not (url := wiki.get(name)):
        return "没有找到该角色的角评，可能是学生名称错误或者插件还未收录该角色角评"
    return MessageSegment.image(await db_get(url, True))


async def db_wiki_raid(raid_id, servers=None, is_wiki=False, terrain=None):
    if not servers:
        servers = [0, 1]
    wiki = (await db_get_wiki_data())["raid"]

    if not (boss := wiki.get(str(raid_id))):
        return "没有找到该总力战Boss"

    terrain_raid = None
    if terrain:
        if t := boss["terrains"].get(
            recover_alia(terrain, await db_get_terrain_alias()),
        ):
            terrain_raid = t
        else:
            return "还没有进行过该环境的总力战"

    img = []
    if is_wiki:
        if not (wiki_url := boss.get("wiki")):
            return "该总力战Boss暂无机制介绍"
        img.append(wiki_url)
    else:
        img_ = [terrain_raid] if terrain_raid else list(boss["terrains"].values())
        for i in img_:
            for s in servers:
                img.append(i[s])

    return [
        MessageSegment.image(x)
        for x in await asyncio.gather(*[db_get(x, True) for x in img])
    ]


async def db_wiki_event(event_id):
    event_id = str(event_id)
    wiki = (await db_get_wiki_data())["event"]
    if not (ev := wiki.get(event_id)):
        return "没有找到该活动"
    return [
        MessageSegment.image(x)
        for x in await asyncio.gather(*[db_get(x, True) for x in ev])
    ]


async def db_wiki_time_atk(raid_id):
    if raid_id >= 1000:
        raid_id = int(raid_id / 1000)
    wiki = (await db_get_wiki_data())["time_atk"]
    if raid_id > len(wiki):
        return f"没有找到该综合战术考试（目前共有{len(wiki)}个综合战术考试）"
    raid_id -= 1

    return MessageSegment.image(await db_get(wiki[raid_id], True))  # type: ignore


async def db_wiki_craft():
    wiki = (await db_get_wiki_data())["craft"]
    return [
        MessageSegment.image(x)
        for x in await asyncio.gather(*[db_get(y, True) for y in wiki])
    ]


async def db_wiki_furniture():
    wiki = (await db_get_wiki_data())["furniture"]
    return [
        MessageSegment.image(x)
        for x in await asyncio.gather(*[db_get(y, True) for y in wiki])
    ]


async def db_global_future(
    date: Optional[datetime.datetime] = None,
    num=1,
    all_img=False,
):
    data = (await db_get_wiki_data())["global_future"]
    img = cast(bytes, await db_get(data["img"], True))

    if all_img:
        return MessageSegment.image(img)

    compare_date = date or datetime.datetime.now()
    index = -1
    for i, v in enumerate(parts := data["parts"]):
        start, end = [datetime.datetime.strptime(x, "%Y/%m/%d") for x in v["date"]]
        if start <= compare_date < end:
            index = i

    if not date:
        index += 1

    if index == -1:
        return "没有找到符合日期的部分"

    sliced_parts = parts[index : index + num]

    if (pl := len(sliced_parts)) < num:
        return f"抱歉，目前后面还没有这么长的前瞻列表……（目前后面还有 {pl} 个）"

    banner_start, banner_end = data["banner"]
    pos_start = sliced_parts[0]["part"][0]
    pos_end = sliced_parts[-1]["part"][1]

    img = BuildImage.open(BytesIO(img))
    width = img.width
    banner = img.crop((0, banner_start, width, banner_end))
    content = img.crop((0, pos_start, width, pos_end))

    bg = (
        BuildImage.new("RGB", (width, banner.height + content.height))
        .paste(banner)
        .paste(content, (0, banner.height))
    )
    return MessageSegment.image(bg.save("PNG"))



import asyncio
from typing import TYPE_CHECKING

from nonebot import on_command
from nonebot.adapters.onebot.v11 import Message
from nonebot.internal.matcher import Matcher
from nonebot.log import logger
from nonebot.params import CommandArg

from ..data.bawiki import db_wiki_time_atk
from ..data.schaledb import find_current_event, schale_get_common
from ..util import splice_msg

if TYPE_CHECKING:
    from . import HelpList

help_list: "HelpList" = [
    {
        "func": "综合战术考试一图流",
        "trigger_method": "指令",
        "trigger_condition": "ba综合战术考试",
        "brief_des": "查询综合战术考试攻略图",
        "detail_des": (
            "发送当前或指定综合战术考试一图流攻略图\n"
            "图片作者 B站@夜猫咪喵喵猫\n"
            " \n"
            "指令默认发送日服和国际服当前的综合战术考试攻略\n"
            "指令后面跟`日`或`j`开头的文本代表查询日服当前综合战术考试攻略，带以`国`或`g`开头的文本同理\n"
            "跟整数则代表指定第几个综合战术考试\n"
            " \n"
            "p.s. 综合战术考试 和 合同火力演习 其实是一个东西，国际服叫前者，日服叫后者～\n"
            " \n"
            "可以用这些指令触发：\n"
            "- <ft color=(238,120,0)>ba综合战术考试</ft>\n"
            "- <ft color=(238,120,0)>ba合同火力演习</ft>\n"
            "- <ft color=(238,120,0)>ba战术考试</ft>\n"
            "- <ft color=(238,120,0)>ba火力演习</ft>\n"
            " \n"
            "指令示例：\n"
            "- <ft color=(238,120,0)>ba综合战术考试</ft>\n"
            "- <ft color=(238,120,0)>ba综合战术考试 日</ft>\n"
            "- <ft color=(238,120,0)>ba综合战术考试 8</ft>"
        ),
    },
]


cmd_time_atk_wiki = on_command("ba综合战术考试", aliases={"ba合同火力演习", "ba战术考试", "ba火力演习"})


@cmd_time_atk_wiki.handle()
async def _(matcher: Matcher, cmd_arg: Message = CommandArg()):
    arg = cmd_arg.extract_plain_text().lower().strip()

    server = []
    if arg.startswith(("日", "j")) or not arg:
        server.append(0)
    if arg.startswith(("国", "g")) or not arg:
        server.append(1)

    events = []
    if server:
        try:
            common = await schale_get_common()
            for s in server:
                raid = common["regions"][s]["current_raid"]
                if (r := find_current_event(raid)) and (raid := r[0]["raid"]) >= 1000:
                    events.append(raid)
        except Exception:
            logger.exception("获取当前综合战术考试失败")
            await matcher.finish("获取当前综合战术考试失败")

        if not events:
            await matcher.finish("当前服务器没有正在进行的综合战术考试")

    else:
        if (not str(arg).isdigit()) or ((arg := int(arg)) < 1):
            await matcher.finish("综合战术考试ID需为整数，从1开始，代表第1个综合战术考试")
        events.append(arg)

    try:
        ret = await asyncio.gather(*[db_wiki_time_atk(x) for x in events])
    except Exception:
        logger.exception("获取综合战术考试wiki出错")
        await matcher.finish("获取图片出错，请检查后台输出")

    await matcher.finish(splice_msg(ret))



from typing import TYPE_CHECKING

from nonebot import on_command
from nonebot.adapters.onebot.v11 import Message, MessageSegment
from nonebot.internal.matcher import Matcher
from nonebot.log import logger
from nonebot.params import CommandArg

from ..config import config
from ..data.bawiki import db_get_extra_l2d_list, recover_stu_alia, schale_to_gamekee
from ..data.gamekee import game_kee_get_stu_cid_li, game_kee_grab_l2d
from ..data.schaledb import draw_fav_li, schale_get_stu_dict
from ..util import async_req

if TYPE_CHECKING:
    from . import HelpList

help_list: "HelpList" = [
    {
        "func": "羁绊查询",
        "trigger_method": "指令",
        "trigger_condition": "ba羁绊",
        "brief_des": "查询学生解锁L2D需求的羁绊等级",
        "detail_des": (
            "使用学生名称查询该学生解锁L2D看板需求的羁绊等级以及L2D预览，"
            "或者使用羁绊等级级数查询哪些学生达到该等级时解锁L2D看板\n"
            "使用学生名称查询时支持部分学生别名\n"
            " \n"
            "可以用这些指令触发：\n"
            "- <ft color=(238,120,0)>ba羁绊</ft>\n"
            "- <ft color=(238,120,0)>ba好感度</ft>\n"
            "- <ft color=(238,120,0)>bal2d</ft>\n"
            "- <ft color=(238,120,0)>balive2d</ft>\n"
            " \n"
            "指令示例：\n"
            "- <ft color=(238,120,0)>ba羁绊 xcw</ft>\n"
            "- <ft color=(238,120,0)>ba羁绊 9</ft>"
        ),
    },
]


cmd_fav = on_command(
    "ba好感度",
    aliases={"ba羁绊", "bal2d", "baL2D", "balive2d", "baLive2D"},
)


@cmd_fav.handle()
async def _(matcher: Matcher, cmd_arg: Message = CommandArg()):
    async def get_l2d(stu_name):
        if r := (await db_get_extra_l2d_list()).get(stu_name):
            return [f"{config.ba_bawiki_db_url}{x}" for x in r]

        return await game_kee_grab_l2d((await game_kee_get_stu_cid_li()).get(stu_name))

    arg = cmd_arg.extract_plain_text().strip()
    if not arg:
        await matcher.finish("请提供学生名称或所需的羁绊等级")

    # 好感度等级
    if arg.isdigit():
        arg = int(arg)
        if arg > 9:
            await matcher.finish("学生解锁L2D最高只需要羁绊等级9")
        if arg < 1:
            await matcher.finish("学生解锁L2D最低只需要羁绊等级1")

        try:
            p = await draw_fav_li(arg)
        except Exception:
            logger.exception("绘制图片出错")
            await matcher.finish("绘制图片出错，请检查后台输出")

        await matcher.finish(p)

    # 学生名称
    arg = await recover_stu_alia(arg)

    try:
        ret = await schale_get_stu_dict()
    except Exception:
        logger.exception("获取学生列表出错")
        await matcher.finish("获取学生列表表出错，请检查后台输出")

    if stu := ret.get(arg):
        if not (lvl := stu["MemoryLobby"]):
            await matcher.finish("该学生没有L2D")

        im = MessageSegment.text(f'{stu["Name"]} 在羁绊等级 {lvl[0]} 时即可解锁L2D\nL2D预览：')
        if p := await get_l2d(await schale_to_gamekee(arg)):
            im += [MessageSegment.image(await async_req(x, raw=True)) for x in p]
        else:
            im += (
                "没找到该学生的L2D看板\n"
                "可能原因：\n"
                "- GameKee页面爬取不到角色L2D图片\n"
                "- GameKee和插件没有收录该学生的L2D\n"
            )
        await matcher.finish(im)

    await matcher.finish("未找到学生")



from nonebot import require
from nonebot.plugin import PluginMetadata

require("nonebot_plugin_apscheduler")
require("nonebot_plugin_htmlrender")

from .command import load_commands  # noqa: E402
from .config import Cfg as Cfg  # noqa: E402
from .help import extra, register_help_cmd, usage  # noqa: E402

__version__ = "0.8.3"
__plugin_meta__ = PluginMetadata(
    name="BAWiki",
    description="碧蓝档案Wiki插件",
    usage=usage,
    homepage="https://github.com/lgc-NB2Dev/nonebot-plugin-bawiki",
    type="application",
    config=Cfg,
    supported_adapters={"~onebot.v11"},
    extra={"License": "MIT", "Author": "student_2333"},
)

if extra:
    __plugin_meta__.extra.update(extra)

register_help_cmd()
load_commands()



import asyncio
from argparse import Namespace
from typing import TYPE_CHECKING

from nonebot import on_shell_command
from nonebot.exception import ParserExit
from nonebot.internal.matcher import Matcher
from nonebot.log import logger
from nonebot.params import ShellCommandArgs
from nonebot.rule import ArgumentParser

from ..data.bawiki import db_get_raid_alias, db_get_terrain_alias, db_wiki_raid
from ..data.schaledb import find_current_event, schale_get_common
from ..util import recover_alia, splice_msg

if TYPE_CHECKING:
    from . import HelpList

help_list: "HelpList" = [
    {
        "func": "总力战一图流",
        "trigger_method": "指令",
        "trigger_condition": "ba总力战",
        "brief_des": "查询总力战推荐配队/Boss机制",
        "detail_des": (
            "发送当前或指定总力战Boss的配队/机制一图流攻略图\n"
            "支持部分Boss别名\n"
            "图片作者 B站@夜猫咪喵喵猫\n"
            " \n"
            "使用 <ft color=(238,120,0)>ba总力战 -h</ft> 查询指令用法\n"
            " \n"
            "指令示例：\n"
            "- <ft color=(238,120,0)>ba总力战</ft>（日服&国际服当前总力战Boss配队攻略）\n"
            "- <ft color=(238,120,0)>ba总力战 -s j</ft>（日服当前总力战Boss配队攻略）\n"
            "- <ft color=(238,120,0)>ba总力战 -s j -w</ft>（日服当前总力战Boss机制图）\n"
            "- <ft color=(238,120,0)>ba总力战 寿司</ft>（Kaiten FX Mk.0 配队攻略）\n"
            "- <ft color=(238,120,0)>ba总力战 寿司 -t 屋外</ft>（Kaiten FX Mk.0 屋外战配队攻略）"
        ),
    },
]


raid_wiki_parser = ArgumentParser("ba总力战")
raid_wiki_parser.add_argument(
    "name",
    nargs="?",
    default=None,
    help="总力战Boss名称，不指定默认取当前服务器总力战Boss",
)
raid_wiki_parser.add_argument(
    "-s",
    "--server",
    nargs="*",
    help="服务器名称，`j`或`日`代表日服，`g`或`国`代表国际服，可指定多个，默认全选",
    default=["j", "g"],
)
raid_wiki_parser.add_argument("-t", "--terrain", help="指定总力战环境，不指定默认全选，不带Boss名称该参数无效")
raid_wiki_parser.add_argument(
    "-w",
    "--wiki",
    action="store_true",
    help="发送该总力战Boss的技能机制而不是配队推荐",
)

cmd_raid_wiki = on_shell_command("ba总力战", parser=raid_wiki_parser)


@cmd_raid_wiki.handle()
async def _(matcher: Matcher, foo: ParserExit = ShellCommandArgs()):
    im = ""
    if foo.status != 0:
        im = "参数错误\n"
    await matcher.finish(f"{im}{foo.message}")


@cmd_raid_wiki.handle()
async def _(matcher: Matcher, args: Namespace = ShellCommandArgs()):
    if not args.server:
        await matcher.finish("请指定server参数")

    server = set()
    for s in args.server:
        if ("日" in s) or ("j" in s):
            server.add(0)
        elif ("国" in s) or ("g" in s):
            server.add(1)
    server = list(server)
    server.sort()

    tasks = []
    if not args.name:
        try:
            common = await schale_get_common()
            for s in server:
                raid = common["regions"][s]["current_raid"]
                if (r := find_current_event(raid)) and (raid := r[0]["raid"]) < 1000:
                    tasks.append(
                        db_wiki_raid(raid, [s], args.wiki, r[0].get("terrain")),
                    )
        except Exception:
            logger.exception("获取当前总力战失败")
            await matcher.finish("获取当前总力战失败")

        if not tasks:
            await matcher.finish("目前服务器没有正在进行的总力战，请手动指定")
    else:
        tasks.append(
            db_wiki_raid(
                recover_alia(args.name, await db_get_raid_alias()),
                server,
                args.wiki,
                (
                    recover_alia(args.terrain, await db_get_terrain_alias())
                    if args.terrain
                    else None
                ),
            ),
        )

    try:
        ret = await asyncio.gather(*tasks)
    except Exception:
        logger.exception("获取总力战wiki失败")
        await matcher.finish("获取图片失败，请检查后台输出")

    await matcher.finish(splice_msg(ret))



from typing import Optional

from nonebot import get_driver
from pydantic import BaseModel, validator


class Cfg(BaseModel):
    ba_proxy: Optional[str] = None
    ba_gacha_cool_down: int = 0
    ba_voice_use_card: bool = False

    ba_gamekee_url: str = "https://ba.gamekee.com/"
    ba_schale_url: str = "https://schale.lgc.cyberczy.xyz/"
    ba_schale_mirror_url: str = "https://schale.lgc.cyberczy.xyz/"
    ba_bawiki_db_url: str = "https://bawiki.lgc.cyberczy.xyz/"
    ba_arona_api_url: str = "https://arona.diyigemt.com/"
    ba_arona_cdn_url: str = "https://arona.cdn.diyigemt.com/"

    ba_clear_req_cache_interval: int = 3
    ba_auto_clear_arona_cache: bool = True

    @validator(
        "ba_gamekee_url",
        "ba_schale_url",
        "ba_schale_mirror_url",
        "ba_bawiki_db_url",
        "ba_arona_api_url",
        "ba_arona_cdn_url",
    )
    def _(cls, val: str):  # noqa: N805
        if not (val.startswith(("https://", "http://")) and val.endswith("/")):
            raise ValueError('自定义的 URL 请以 "http(s)://" 开头，以 "/" 结尾')
        return val


config = Cfg.parse_obj(get_driver().config.dict())



import shutil
from typing import TYPE_CHECKING

import anyio
from nonebot import on_command
from nonebot.internal.matcher import Matcher
from nonebot.permission import SUPERUSER

from ..resource import CACHE_PATH
from ..util import clear_req_cache

if TYPE_CHECKING:
    from . import HelpList

help_list: "HelpList" = [
    {
        "func": "清空缓存",
        "trigger_method": "超级用户 指令",
        "trigger_condition": "ba清空缓存",
        "brief_des": "清空插件请求缓存",
        "detail_des": (
            "手动清空插件请求网络缓存下来的数据，如API返回的数据\n"
            "注：该指令只能由<ft color=(238,120,0)>超级用户</ft>触发\n"
            " \n"
            "可以用这些指令触发：\n"
            "- <ft color=(238,120,0)>ba清空缓存</ft>\n"
            "- <ft color=(238,120,0)>ba清除缓存</ft>"
        ),
    },
]


cmd_clear_cache = on_command("ba清空缓存", aliases={"ba清除缓存"}, permission=SUPERUSER)


@cmd_clear_cache.handle()
async def _(matcher: Matcher):
    clear_req_cache()

    async for path in anyio.Path(CACHE_PATH).iterdir():
        await path.unlink() if path.is_file() else shutil.rmtree(path)

    await matcher.finish("缓存已清空～")



import importlib
from pathlib import Path
from typing import List, TypedDict

from nonebot import logger
from pypinyin import lazy_pinyin


class HelpDict(TypedDict):
    func: str
    trigger_method: str
    trigger_condition: str
    brief_des: str
    detail_des: str


HelpList = List[HelpDict]

help_list: HelpList = []


def sort_help():
    help_list.sort(key=lambda x: "".join(lazy_pinyin(x["func"])))


def append_and_sort_help(help_dict: HelpDict):
    help_list.append(help_dict)
    sort_help()


def load_commands():
    for module in Path(__file__).parent.iterdir():
        if module.name.startswith("_"):
            continue

        module = importlib.import_module(f".{module.stem}", __package__)
        assert module

        if not hasattr(module, "help_list"):
            logger.warning(f"Command module `{module.__name__}` has no `help_list`")
        else:
            help_list.extend(module.help_list)

    sort_help()



import datetime
from typing import TYPE_CHECKING

from nonebot import on_command
from nonebot.adapters.onebot.v11 import Message
from nonebot.internal.matcher import Matcher
from nonebot.params import CommandArg

from ..data.bawiki import db_global_future

if TYPE_CHECKING:
    from . import HelpList

help_list: "HelpList" = [
    {
        "func": "国际服千里眼",
        "trigger_method": "指令",
        "trigger_condition": "ba千里眼",
        "brief_des": "查询国际服未来的卡池与活动",
        "detail_des": (
            "发送当前或指定日期的国际服未来卡池与活动列表\n"
            "图片作者 B站@夜猫咪喵喵猫\n"
            " \n"
            "参数可以指定起始日期以及列表个数，但同时指定时请将日期放在列表个数前面，以空格分隔\n"
            "参数中含有`全`或`a`时会发送整张前瞻图\n"
            " \n"
            "日期格式可以为（Y代表4位数年，m代表月，d代表日）：\n"
            "- <ft color=(238,120,0)>Y/m/d</ft>；<ft color=(238,120,0)>m/d</ft>\n"
            "- <ft color=(238,120,0)>Y-m-d</ft>；<ft color=(238,120,0)>m-d</ft>\n"
            "- <ft color=(238,120,0)>Y年m月d日</ft>；<ft color=(238,120,0)>m月d日</ft>\n"
            " \n"
            "可以用这些指令触发：\n"
            "- <ft color=(238,120,0)>ba国际服千里眼</ft>\n"
            "- <ft color=(238,120,0)>ba千里眼</ft>\n"
            "- <ft color=(238,120,0)>ba国际服前瞻</ft>\n"
            "- <ft color=(238,120,0)>ba前瞻</ft>\n"
            " \n"
            "指令示例：\n"
            "- <ft color=(238,120,0)>ba千里眼</ft>\n"
            "- <ft color=(238,120,0)>ba千里眼 all</ft>\n"
            "- <ft color=(238,120,0)>ba千里眼 3</ft>\n"
            "- <ft color=(238,120,0)>ba千里眼 11/15</ft>\n"
            "- <ft color=(238,120,0)>ba千里眼 11/15 3</ft>"
        ),
    },
]


cmd_global_future = on_command("ba国际服千里眼", aliases={"ba千里眼", "ba国际服前瞻", "ba前瞻"})


@cmd_global_future.handle()
async def _(matcher: Matcher, arg: Message = CommandArg()):
    args = arg.extract_plain_text().strip()
    if "全" in args or "a" in args:
        await matcher.finish(await db_global_future(all_img=True))

    args = args.split()
    num = 1
    date = None
    if (args_len := len(args)) == 1:
        if args[0].isdigit():
            num = args[0]
        else:
            date = args[0]
    elif args_len > 1:
        date = args[0].strip()
        num = args[-1].strip()

    if date:
        parsed_date = None
        for f in ["%Y/%m/%d", "%Y-%m-%d", "%Y年%m月%d日", "%m/%d", "%m-%d", "%m月%d日"]:
            try:
                parsed_date = datetime.datetime.strptime(date.replace(" ", ""), f)
                break
            except ValueError:
                pass
        if not parsed_date:
            await matcher.finish("日期格式不正确！")
        date = parsed_date
        if date.year == 1900:
            now = datetime.datetime.now().replace(
                hour=0,
                minute=0,
                second=0,
                microsecond=0,
            )
            date = date.replace(year=now.year)
            if date < now:
                date = date.replace(year=now.year + 1)

    if isinstance(num, str) and ((not num.isdigit()) or (num := int(num)) < 1):
        await matcher.finish("前瞻项目数量格式不正确！")

    await matcher.finish(await db_global_future(date or None, num))



from typing import TYPE_CHECKING

from nonebot import on_command
from nonebot.adapters.onebot.v11 import Message
from nonebot.internal.matcher import Matcher
from nonebot.log import logger
from nonebot.params import CommandArg

from ..data.bawiki import db_wiki_stu, recover_stu_alia

if TYPE_CHECKING:
    from . import HelpList

help_list: "HelpList" = [
    {
        "func": "学生评价",
        "trigger_method": "指令",
        "trigger_condition": "ba角评",
        "brief_des": "查询学生角评一图流",
        "detail_des": (
            "发送一张指定角色的评价图\n"
            "支持部分学生别名\n"
            "角评图作者 B站@夜猫咪喵喵猫\n"
            " \n"
            "可以使用 `all` / `总览` / `全部` 参数 查看全学生角评一图流\n"
            " \n"
            "可以用这些指令触发：\n"
            "- <ft color=(238,120,0)>ba学生评价</ft>\n"
            "- <ft color=(238,120,0)>ba角评</ft>\n"
            " \n"
            "指令示例：\n"
            "- <ft color=(238,120,0)>ba学生评价 白子</ft>\n"
            "- <ft color=(238,120,0)>ba角评 xcw</ft>\n"
            "- <ft color=(238,120,0)>ba角评 总览</ft>"
        ),
    },
]


cmd_stu_rank = on_command("ba学生评价", aliases={"ba角评"})


@cmd_stu_rank.handle()
async def _(matcher: Matcher, cmd_arg: Message = CommandArg()):
    arg = cmd_arg.extract_plain_text().strip()
    if not arg:
        await matcher.finish("请提供学生名称")

    if arg == "总览" or arg == "全部" or arg.lower() == "all":
        arg = "all"
    else:
        arg = await recover_stu_alia(arg)

    try:
        im = await db_wiki_stu(arg)
    except Exception:
        logger.exception("获取角评出错")
        await matcher.finish("获取角评出错，请检查后台输出")

    await matcher.finish(im)



import re

from nonebot.internal.adapter import Message
from nonebot.internal.matcher import Matcher
from nonebot.params import CommandArg

from ..command import help_list

usage = "使用指令 `ba帮助` 查询插件功能帮助"
extra = None

FT_START_REGEX = r"<ft color=\((\d+),(\d+),(\d+)\)>"
FT_END_REGEX = r"</ft>"


async def help_handle(matcher: Matcher, arg_msg: Message = CommandArg()):
    arg = arg_msg.extract_plain_text().strip()

    if not arg:
        cmd_list = "\n".join(
            f"▶ {k['func']} ({k['trigger_method']}：{k['trigger_condition']}) - {k['brief_des']}"
            for k in help_list
        )
        await matcher.finish(
            f"目前插件支持的功能：\n"
            f"\n"
            f"{cmd_list}\n"
            f"\n"
            f"Tip: 使用指令 `ba帮助 <功能名>` 查看某功能详细信息",
        )

    arg_lower = arg.lower()
    func = next(
        (
            x
            for x in help_list
            if (
                (arg_lower in x["func"].lower())
                or (arg_lower in x["trigger_condition"].lower())
            )
        ),
        None,
    )
    if not func:
        await matcher.finish(f"未找到功能 `{arg}`")

    # 去掉 <ft> 富文本标签
    detail_des = re.sub(FT_START_REGEX, "", func["detail_des"])
    detail_des = re.sub(FT_END_REGEX, "", detail_des)

    # 缩进
    detail_des = "    ".join(detail_des.splitlines(keepends=True)).strip()

    await matcher.finish(
        f"▶ 功能：{func['func']}\n"
        f"\n"
        f'▷ 触发方式：{func["trigger_method"]}\n'
        f'▷ 触发条件：{func["trigger_condition"]}\n'
        f'▷ 简要描述：{func["brief_des"]}\n'
        f"▷ 详细描述：\n"
        f"    {detail_des}",
    )



import asyncio
import math
import time
from datetime import datetime
from io import BytesIO
from typing import Any, Dict, List, Literal, cast, overload

from nonebot import logger
from nonebot.adapters.onebot.v11 import MessageSegment
from nonebot_plugin_htmlrender import get_new_page
from PIL import Image, ImageFilter
from PIL.Image import Resampling
from pil_utils import BuildImage, text2image
from playwright.async_api import Page, ViewportSize

from ..config import config
from ..resource import RES_CALENDER_BANNER, RES_GRADIENT_BG
from ..util import async_req, img_invert_rgba, parse_time_delta

PAGE_KWARGS = {
    "is_mobile": True,
    "viewport": ViewportSize(width=767, height=800),
}


@overload
async def schale_get(suffix: str, raw: Literal[False] = False, **kwargs) -> Any:
    ...


@overload
async def schale_get(suffix: str, raw: Literal[True] = True, **kwargs) -> bytes:
    ...


async def schale_get(suffix, raw=False, **kwargs):
    return await async_req(f"{config.ba_schale_url}{suffix}", raw=raw, **kwargs)  # type: ignore


async def schale_get_stu_data(loc: str = "cn") -> List[Dict[str, Any]]:
    return await schale_get(f"data/{loc}/students.min.json")


async def schale_get_common() -> Dict[str, Any]:
    return await schale_get("data/common.min.json")


async def schale_get_localization(loc: str = "cn") -> Dict[str, Any]:
    return cast(dict, await schale_get(f"data/{loc}/localization.min.json"))


async def schale_get_raids(loc: str = "cn") -> Dict[str, Any]:
    return cast(dict, await schale_get(f"data/{loc}/raids.min.json"))


async def schale_get_stu_dict(key="Name"):
    return {x[key]: x for x in await schale_get_stu_data()}


async def schale_get_stu_info(stu):
    async with cast(Page, get_new_page(**PAGE_KWARGS)) as page:
        await page.goto(
            f"{config.ba_schale_mirror_url}?chara={stu}",
            timeout=60 * 1000,
            wait_until="networkidle",
        )

        # 进度条拉最大
        await page.add_script_tag(content="utilStuSetAllProgressMax();")

        return await page.screenshot(full_page=True)


async def schale_calender(server):
    return MessageSegment.image(
        await schale_get_calender(
            server,
            *(
                await asyncio.gather(
                    schale_get_stu_data(),
                    schale_get_common(),
                    schale_get_localization(),
                    schale_get_raids(),
                )
            ),
        ),
    )


def find_current_event(ev, now=None):
    if not now:
        now = datetime.now()
    for _e in ev:
        _start = datetime.fromtimestamp(_e["start"])
        _end = datetime.fromtimestamp(_e["end"])
        if _start <= now < _end:
            _remain = _end - now
            return _e, _start, _end, _remain
    return None


async def schale_get_calender(server, students, common, localization, raids):
    students = {x["Id"]: x for x in students}

    region = common["regions"][server]
    now = datetime.now()

    pic_bg = BuildImage.new("RGBA", (1400, 640), (255, 255, 255, 70))

    def format_time(_start, _end, _remain):
        dd, hh, mm, ss = parse_time_delta(_remain)
        return (
            f"{_start} ~ {_end} | "
            f"剩余 [b][color=#fc6475]{dd}天 {hh:0>2d}:{mm:0>2d}:{ss:0>2d}[color=#fc6475][/b]"
        )

    async def draw_gacha():
        pic = pic_bg.copy().draw_text(
            (25, 25, 1375, 150),
            "特选招募",
            weight="bold",
            max_fontsize=80,
        )
        c_gacha = region["current_gacha"]
        if r := find_current_event(c_gacha):
            g = r[0]
            t = format_time(*(r[1:]))
            pic = pic.paste(
                ti := text2image(
                    t,
                    (255, 255, 255, 0),
                    fontsize=45,
                    max_width=1350,
                    align="center",
                ),
                (int((1400 - ti.width) / 2), 150),
                True,
            )
            stu = [students[x] for x in g["characters"]]

            async def process_avatar(s):
                return (
                    BuildImage.open(
                        BytesIO(
                            await schale_get(
                                f'images/student/collection/{s["CollectionTexture"]}.webp',
                                raw=True,
                            ),
                        ),
                    )
                    .resize((300, 340))
                    .paste(
                        BuildImage.new("RGBA", (300, 65), (255, 255, 255, 120)),
                        (0, 275),
                        True,
                    )
                    .convert("RGB")
                    .circle_corner(25)
                    .draw_text((0, 275, 300, 340), s["Name"], max_fontsize=50)
                )

            avatars = await asyncio.gather(*[process_avatar(x) for x in stu])
            ava_len = len(avatars)
            x_index = int((1400 - (300 + 25) * ava_len + 25) / 2)

            for p in avatars:
                pic = pic.paste(p, (x_index, 250), True)
                x_index += p.width + 25

            return pic
        return None

    async def draw_event():
        pic = pic_bg.copy().draw_text(
            (25, 25, 1375, 150),
            "当前活动",
            weight="bold",
            max_fontsize=80,
        )
        c_event = region["current_events"]
        if r := find_current_event(c_event):
            g = r[0]
            t = format_time(*(r[1:]))
            pic = pic.paste(
                ti := text2image(
                    t,
                    (255, 255, 255, 0),
                    fontsize=45,
                    max_width=1350,
                    align="center",
                ),
                (int((1400 - ti.width) / 2), 150),
                True,
            )
            ev = g["event"]
            ev_name = ""
            if ev >= 10000:
                ev_name = " (复刻)"
                ev %= 10000
            ev_name = localization["EventName"][str(ev)] + ev_name

            ev_bg, ev_img = await asyncio.gather(
                schale_get(f"images/campaign/Campaign_Event_{ev}_Normal.png", True),
                schale_get(
                    f"images/eventlogo/Event_{ev}_{'Tw' if server else 'Jp'}.png",
                    True,
                ),
            )

            ev_bg = (
                BuildImage.open(BytesIO(ev_bg))
                .convert("RGBA")
                .resize_height(340)
                .filter(ImageFilter.GaussianBlur(3))
            )
            ev_bg = (
                ev_bg.paste(
                    BuildImage.open(BytesIO(ev_img))
                    .convert("RGBA")
                    .resize(
                        (ev_bg.width, ev_bg.height - 65),
                        keep_ratio=True,
                        inside=True,
                        bg_color=(255, 255, 255, 0),
                    ),
                    alpha=True,
                )
                .paste(
                    BuildImage.new("RGBA", (ev_bg.width, 65), (255, 255, 255, 120)),
                    (0, ev_bg.height - 65),
                    True,
                )
                .convert("RGB")
                .circle_corner(25)
                .draw_text(
                    (0, ev_bg.height - 65, ev_bg.width, ev_bg.height),
                    ev_name,
                    max_fontsize=50,
                )
            )
            return pic.paste(ev_bg, (int((pic.width - ev_bg.width) / 2), 250), True)
        return None

    async def draw_raid():
        pic = pic_bg.copy()
        if r := find_current_event(region["current_raid"]):
            ri = r[0]
            t = format_time(*(r[1:]))
            pic = pic.paste(
                ti := text2image(
                    t,
                    (255, 255, 255, 0),
                    fontsize=45,
                    max_width=1350,
                    align="center",
                ),
                (int((1400 - ti.width) / 2), 150),
                True,
            )

            tp = "TimeAttack" if (time_atk := (ri["raid"] >= 1000)) else "Raid"
            raid = {x["Id"]: x for x in raids[tp]}
            c_ri = raid[ri["raid"]]
            pic = pic.draw_text(
                (25, 25, 1375, 150),
                localization["StageType"][tp],
                weight="bold",
                max_fontsize=80,
            )

            if time_atk:
                tk_bg = {
                    "Shooting": "TimeAttack_SlotBG_02",
                    "Defense": "TimeAttack_SlotBG_01",
                    "Destruction": "TimeAttack_SlotBG_03",
                }
                bg_url = f'images/timeattack/{tk_bg[c_ri["DungeonType"]]}'
                fg_url = f'images/enemy/{c_ri["Icon"]}.png'
            else:
                bg_url = f'images/raid/Boss_Portrait_{c_ri["PathName"]}_LobbyBG'
                if len(c_ri["Terrain"]) > 1 and ri["terrain"] == c_ri["Terrain"][1]:
                    bg_url += f'_{ri["terrain"]}'
                fg_url = f'images/raid/Boss_Portrait_{c_ri["PathName"]}_Lobby.png'
            bg_url += ".png"
            terrain = c_ri["Terrain"] if time_atk else ri["terrain"]

            color_map = {
                "LightArmor": (167, 12, 25),
                "Explosion": (167, 12, 25),
                "HeavyArmor": (178, 109, 31),
                "Pierce": (178, 109, 31),
                "Unarmed": (33, 111, 156),
                "Mystic": (33, 111, 156),
                "Normal": (115, 115, 115),
            }
            atk_color = color_map[
                c_ri["BulletType" if time_atk else "BulletTypeInsane"]
            ]
            def_color = color_map[c_ri["ArmorType"]]

            c_bg, c_fg, icon_def, icon_atk, icon_tr = await asyncio.gather(
                *[
                    schale_get(bg_url, True),
                    schale_get(fg_url, True),
                    schale_get("images/ui/Type_Defense_s.png", True),
                    schale_get("images/ui/Type_Attack_s.png", True),
                    schale_get(f"images/ui/Terrain_{terrain}.png", True),
                ],
            )

            icon_def = (
                BuildImage.new("RGBA", (64, 64), def_color)
                .paste(
                    BuildImage.open(BytesIO(icon_def))
                    .convert("RGBA")
                    .resize_height(48),
                    (8, 8),
                    True,
                )
                .circle()
            )
            icon_atk = (
                BuildImage.new("RGBA", (64, 64), atk_color)
                .paste(
                    BuildImage.open(BytesIO(icon_atk))
                    .convert("RGBA")
                    .resize_height(48),
                    (8, 8),
                    True,
                )
                .circle()
            )
            icon_tr = (
                BuildImage.new("RGBA", (64, 64), "#ffffff")
                .paste(
                    img_invert_rgba(Image.open(BytesIO(icon_tr)).convert("RGBA")),
                    (-2, -2),
                    True,
                )
                .circle()
            )

            c_bg = (
                BuildImage.open(BytesIO(c_bg))
                .convert("RGBA")
                .resize_height(340)
                .filter(ImageFilter.GaussianBlur(3))
            )
            c_fg = (
                BuildImage.open(BytesIO(c_fg))
                .convert("RGBA")
                .resize_height(c_bg.height)
            )
            c_bg = (
                c_bg.paste(
                    c_fg,
                    (int((c_bg.width - c_fg.width) / 2), 0),
                    True,
                )
                .paste(
                    BuildImage.new("RGBA", (c_bg.width, 65), (255, 255, 255, 120)),
                    (0, c_bg.height - 65),
                    True,
                )
                .paste(icon_atk, (10, 10), True)
                .paste(icon_def, (10, 79), True)
                .paste(icon_tr, (10, 147), True)
                .convert("RGB")
                .circle_corner(25)
                .draw_text(
                    (0, c_bg.height - 65, c_bg.width, c_bg.height),
                    (
                        localization["TimeAttackStage"][c_ri["DungeonType"]]
                        if time_atk
                        else (c_ri["Name"])
                    ),
                    max_fontsize=50,
                )
            )
            return pic.paste(c_bg, (int((pic.width - c_bg.width) / 2), 250), True)
        return None

    async def draw_birth():
        pic = pic_bg.copy().draw_text(
            (25, 25, 1375, 150),
            "学生生日",
            weight="bold",
            max_fontsize=80,
        )
        now_t = time.mktime(now.date().timetuple())
        now_w = now.weekday()
        this_week_t = now_t - now_w * 86400
        next_week_t = now_t + (7 - now_w) * 86400
        next_next_week_t = next_week_t + 7 * 86400

        birth_this_week = []
        birth_next_week = []
        for s in [x for x in students.values() if x["IsReleased"][server]]:
            birth = time.mktime(
                time.strptime(f'{now.year}/{s["BirthDay"]}', "%Y/%m/%d"),
            )
            if this_week_t <= birth < next_week_t:
                birth_this_week.append(s)
            elif next_week_t <= birth <= next_next_week_t:
                birth_next_week.append(s)

        sort_key = lambda x: x["BirthDay"].split("/")  # noqa: E731
        p_h = 0
        if birth_this_week:
            birth_this_week.sort(key=sort_key)
            p_h += 180

        if birth_next_week:
            birth_next_week.sort(key=sort_key)
            p_h += 180
            if birth_this_week:
                p_h += 10

        if p_h:
            stu_pics = [
                BuildImage.open(BytesIO(x)).convert("RGBA").resize_height(180).circle()
                for x in await asyncio.gather(
                    *[
                        schale_get(
                            f'images/student/icon/{x["CollectionTexture"]}.png',
                            True,
                        )
                        for x in birth_this_week + birth_next_week
                    ],
                )
            ]
            y_index = int((415 - p_h) / 2) + 125
            if birth_this_week:
                x_index = (
                    int((1400 - (len(birth_this_week) * (180 + 10) - 10)) / 2) + 75
                )
                pic = pic.draw_text(
                    (x_index - 165, y_index, x_index, y_index + 180),
                    "本周",
                    max_fontsize=50,
                )
                for s in birth_this_week:
                    pic.paste(stu_pics.pop(0), (x_index, y_index), True).draw_text(
                        (x_index, y_index + 180, x_index + 180, y_index + 220),
                        s["BirthDay"],
                    )
                    x_index += 180 + 10

            if birth_next_week:
                if birth_this_week:
                    y_index += 220 + 10
                x_index = (
                    int((1400 - (len(birth_next_week) * (180 + 10) - 10)) / 2) + 75
                )
                pic = pic.draw_text(
                    (x_index - 165, y_index, x_index, y_index + 180),
                    "下周",
                    max_fontsize=50,
                )
                for s in birth_next_week:
                    pic.paste(stu_pics.pop(0), (x_index, y_index), True).draw_text(
                        (x_index, y_index + 180, x_index + 180, y_index + 220),
                        s["BirthDay"],
                    )
                    x_index += 180 + 10

            return pic
        return None

    img = await asyncio.gather(  # type: ignore
        draw_gacha(),
        draw_event(),
        draw_raid(),
        draw_birth(),
    )
    img: List[BuildImage] = [x for x in img if x]
    if not img:
        img.append(
            pic_bg.copy().draw_text((0, 0, 1400, 640), "没有获取到任何数据", max_fontsize=60),
        )

    bg_w = 1500
    bg_h = 200 + sum([x.height + 50 for x in img])
    bg = (
        BuildImage.new("RGBA", (bg_w, bg_h))
        .paste(RES_CALENDER_BANNER.copy().resize((1500, 150)))
        .draw_text(
            (50, 0, 1480, 150),
            f"SchaleDB丨活动日程丨{localization['ServerName'][str(server)]}",
            max_fontsize=100,
            weight="bold",
            fill="#ffffff",
            halign="left",
        )
        .paste(
            RES_GRADIENT_BG.copy().resize(
                (1500, bg_h - 150),
                resample=Resampling.NEAREST,
            ),
            (0, 150),
        )
    )

    h_index = 200
    for im in img:
        bg.paste(im.circle_corner(10), (50, h_index), True)
        h_index += im.height + 50
    return bg.convert("RGB").save("png")


async def draw_fav_li(lvl):
    try:
        stu_li = [
            x
            for x in await schale_get_stu_data()
            if (x["MemoryLobby"] and x["MemoryLobby"][0] == lvl)
        ]
    except Exception:
        logger.exception("获取schale db学生数据失败")
        return "获取SchaleDB学生数据失败，请检查后台输出"

    if not stu_li:
        return f"没有学生在羁绊等级{lvl}时解锁L2D"

    txt_h = 48
    pic_h = 144
    icon_w = 182
    icon_h = pic_h + txt_h
    line_max_icon = 6

    if (li_len := len(stu_li)) <= line_max_icon:
        line = 1
        length = li_len
    else:
        line = math.ceil(li_len / line_max_icon)
        length = line_max_icon

    img = RES_GRADIENT_BG.copy().resize(
        (icon_w * length, icon_h * line + 5),
        resample=Resampling.NEAREST,
    )

    async def draw_stu(name_, dev_name_, line_, index_):
        left = index_ * icon_w
        top = line_ * icon_h + 5

        ret = await schale_get(
            f"images/student/lobby/Lobbyillust_Icon_{dev_name_}_01.png",
            True,
        )
        icon_img = Image.open(BytesIO(ret)).convert("RGBA")
        img.paste(icon_img, (left, top), True)
        img.draw_text(
            (left, top + pic_h, left + icon_w, top + icon_h),
            name_,
            max_fontsize=25,
            min_fontsize=1,
        )

    task_li = []
    line = 0
    i = 0
    for stu in stu_li:
        if i == line_max_icon:
            i = 0
            line += 1
        task_li.append(draw_stu(stu["Name"], stu["DevName"], line, i))
        i += 1
    await asyncio.gather(*task_li)

    return MessageSegment.text(f"羁绊等级 {lvl} 时解锁L2D的学生有以下这些：") + MessageSegment.image(
        img.save("png"),
    )



import asyncio
import random
from io import BytesIO
from typing import TYPE_CHECKING

from nonebot import on_command
from nonebot.adapters.onebot.v11 import Message, MessageSegment
from nonebot.internal.matcher import Matcher
from nonebot.log import logger
from pil_utils import BuildImage

from ..data.bawiki import MangaDict, db_get_manga
from ..util import async_req

if TYPE_CHECKING:
    from . import HelpList

help_list: "HelpList" = [
    {
        "func": "随机漫画",
        "trigger_method": "指令",
        "trigger_condition": "ba漫画",
        "brief_des": "随机发送一话官推漫画",
        "detail_des": "随机发送一话BA官推漫画\n来源：GameKee",
    },
]


cmd_random_manga = on_command("ba漫画")


@cmd_random_manga.handle()
async def _(matcher: Matcher):
    async def get_pic(url):
        p = await async_req(url, raw=True)
        if url.endswith(".webp"):
            p = BuildImage.open(BytesIO(p)).save_png()
        return p

    try:
        manga: MangaDict = random.choice(await db_get_manga())
        pics = await asyncio.gather(*[get_pic(x) for x in manga["pics"]])
    except Exception:
        logger.exception("获取漫画失败")
        await matcher.finish("获取漫画失败，请检查后台输出")

    await matcher.finish(
        Message()
        + f'{manga["title"]}\n-=-=-=-=-=-=-=-\n{manga["detail"]}'
        + [MessageSegment.image(x) for x in pics],
    )



import shutil
from pathlib import Path

from nonebot import logger
from pil_utils import BuildImage

RES_PATH = Path(__file__).parent / "res"
DATA_PATH = Path.cwd() / "data" / "BAWiki"
CACHE_PATH = DATA_PATH / "cache"

_OLD_CACHE_FOLDER = Path.cwd() / "cache"
_OLD_CACHE_PATH = _OLD_CACHE_FOLDER / "BAWiki"

if not DATA_PATH.exists():
    DATA_PATH.mkdir(parents=True)

if _OLD_CACHE_PATH.exists():
    logger.warning("Moving old cache to new path...")
    shutil.move(str(_OLD_CACHE_PATH), CACHE_PATH)

    if not any(_OLD_CACHE_FOLDER.iterdir()):
        _OLD_CACHE_FOLDER.rmdir()

if not CACHE_PATH.exists():
    CACHE_PATH.mkdir(parents=True)

RES_CALENDER_BANNER = BuildImage.open(RES_PATH / "calender_banner.png")
RES_GRADIENT_BG = BuildImage.open(RES_PATH / "gradient.png")
RES_GACHA_BG = BuildImage.open(RES_PATH / "gacha_bg.png")
RES_GACHA_CARD_BG = BuildImage.open(RES_PATH / "gacha_card_bg.png")
RES_GACHA_CARD_MASK = BuildImage.open(RES_PATH / "gacha_card_mask.png").convert("RGBA")
RES_GACHA_NEW = BuildImage.open(RES_PATH / "gacha_new.png")
RES_GACHA_PICKUP = BuildImage.open(RES_PATH / "gacha_pickup.png")
RES_GACHA_STAR = BuildImage.open(RES_PATH / "gacha_star.png")
RES_GACHA_STU_ERR = BuildImage.open(RES_PATH / "gacha_stu_err.png")



import math
from dataclasses import dataclass
from typing import TYPE_CHECKING, Dict, List, Optional

from nonebot import on_command
from nonebot.adapters.onebot.v11 import (
    GroupMessageEvent,
    Message,
    MessageEvent,
    MessageSegment,
)
from nonebot.internal.matcher import Matcher
from nonebot.log import logger
from nonebot.params import CommandArg

from ..data.bawiki import db_get_gacha_data, db_get_stu_alias
from ..data.gacha import gacha, get_gacha_cool_down, set_gacha_cool_down
from ..data.schaledb import schale_get_stu_dict
from ..util import recover_alia

if TYPE_CHECKING:
    from . import HelpList

help_list: "HelpList" = [
    {
        "func": "模拟抽卡",
        "trigger_method": "指令",
        "trigger_condition": "ba抽卡",
        "brief_des": "模拟抽卡",
        "detail_des": (
            "模拟抽卡\n"
            "可以使用 <ft color=(238,120,0)>ba切换卡池</ft> 指令来切换卡池\n"
            "可以指定抽卡次数，需要在1~90之间，默认10\n"
            " \n"
            "指令示例：\n"
            "- <ft color=(238,120,0)>ba抽卡</ft>\n"
            "- <ft color=(238,120,0)>ba抽卡 20</ft>"
        ),
    },
    {
        "func": "切换卡池",
        "trigger_method": "指令",
        "trigger_condition": "ba切换卡池",
        "brief_des": "设置模拟抽卡的UP池",
        "detail_des": (
            "设置模拟抽卡功能的UP池角色\n"
            "默认从当前数据源UP池中轮流切换\n"
            "当参数为 <ft color=(238,120,0)>常驻</ft> 时，切换到常驻池（没有UP）\n"
            "可以自定义池子UP角色，支持2星与3星角色，参数中学生名称用空格分隔，支持部分学生别名\n"
            " \n"
            "指令示例：\n"
            "- <ft color=(238,120,0)>ba切换卡池</ft>\n"
            "- <ft color=(238,120,0)>ba切换卡池 常驻</ft>\n"
            "- <ft color=(238,120,0)>ba切换卡池 小桃 小绿</ft>"
        ),
    },
]


cmd_change_pool = on_command("ba切换卡池")
cmd_gacha_once = on_command("ba抽卡")


@dataclass()
class GachaPool:
    name: str
    pool: List[int]
    index: Optional[int] = None


gacha_pool_index: Dict[str, GachaPool] = {}


def get_1st_pool(data: dict) -> Optional[GachaPool]:
    if not data:
        return None

    pool_data = data["current_pools"]
    if not pool_data:
        return None

    pool = pool_data[0]
    return GachaPool(name=pool["name"], pool=pool["pool"], index=0)


@cmd_change_pool.handle()
async def _(matcher: Matcher, event: MessageEvent, cmd_arg: Message = CommandArg()):
    arg = cmd_arg.extract_plain_text().strip().lower()
    qq = event.get_user_id()

    if arg:
        if "常驻" in arg:
            current = GachaPool(name="常驻池", pool=[])
        else:
            pool = []
            try:
                stu_li = await schale_get_stu_dict()
                stu_alias = await db_get_stu_alias()
            except Exception:
                logger.exception("获取学生列表或别名失败")
                await matcher.finish("获取学生列表或别名失败，请检查后台输出")

            for i in arg.split():
                if not (stu := stu_li.get(recover_alia(i, stu_alias))):
                    await matcher.finish(f"未找到学生 {i}")
                if stu["StarGrade"] == 1:
                    await matcher.finish("不能UP一星角色")
                pool.append(stu)

            current = GachaPool(
                name=f"自定义卡池（{'、'.join([x['Name'] for x in pool])}）",
                pool=[x["Id"] for x in pool],
            )

    else:
        try:
            gacha_data = await db_get_gacha_data()
        except Exception:
            logger.exception("获取抽卡基本数据失败")
            await matcher.finish("获取抽卡基本数据失败，请检查后台输出")

        pool_data = gacha_data["current_pools"]
        first_pool = get_1st_pool(gacha_data)
        if not first_pool:
            await matcher.finish("当前没有可切换的卡池")

        pool_obj = gacha_pool_index.get(qq) or first_pool
        if not pool_obj:
            await matcher.finish("当前没有UP池可供切换")

        if (index := pool_obj.index) is not None:
            index += 1
            if len(pool_data) <= index:
                index = 0
        else:
            index = 0

        pool = pool_data[index]
        current = GachaPool(name=pool["name"], pool=pool["pool"], index=index)

    gacha_pool_index[qq] = current
    await matcher.finish(f"已切换到卡池 {current.name}")


@cmd_gacha_once.handle()
async def _(matcher: Matcher, event: MessageEvent, cmd_arg: Message = CommandArg()):
    user_id = event.user_id
    group_id = event.group_id if isinstance(event, GroupMessageEvent) else None

    if cool_down := get_gacha_cool_down(user_id, group_id):
        await matcher.finish(f"你先别急，先等 {cool_down} 秒再来抽吧qwq")

    set_gacha_cool_down(user_id, group_id)

    arg = cmd_arg.extract_plain_text().strip().lower()

    gacha_times = 10
    if arg and ((not arg.isdigit()) or (not (1 <= (gacha_times := int(arg)) <= 90))):
        await matcher.finish("请输入有效的抽卡次数，在1~90之间")

    try:
        gacha_data = await db_get_gacha_data()
    except Exception:
        logger.exception("获取抽卡基本数据失败")
        await matcher.finish("获取抽卡基本数据失败，请检查后台输出")

    pool_obj = gacha_pool_index.get(qq := event.get_user_id()) or get_1st_pool(
        gacha_data,
    )
    if not pool_obj:
        await matcher.finish("目前没有UP池，请使用[ba切换卡池]指令来切换到常驻池或组建一个自定义UP池")

    try:
        img = []
        for _ in range(math.ceil(gacha_times / 10)):
            img.append(
                await gacha(
                    qq,
                    10 if gacha_times >= 10 else gacha_times,
                    gacha_data,
                    pool_obj.pool,
                ),
            )
            gacha_times -= 10
    except Exception:
        logger.exception("抽卡错误")
        await matcher.finish("抽卡出错了，请检查后台输出")

    await matcher.finish(MessageSegment.at(qq) + f"当前抽取卡池：{pool_obj.name}" + img)



import random
from typing import TYPE_CHECKING

from nonebot import on_command
from nonebot.adapters.onebot.v11 import MessageSegment
from nonebot.internal.matcher import Matcher
from nonebot.log import logger

from ..data.bawiki import db_get, db_get_emoji

if TYPE_CHECKING:
    from . import HelpList

help_list: "HelpList" = [
    {
        "func": "抽表情",
        "trigger_method": "指令",
        "trigger_condition": "ba表情",
        "brief_des": "随机发送一个国际服社团聊天表情",
        "detail_des": "随机发送一个国际服社团聊天表情\n来源：解包",
    },
]


cmd_random_emoji = on_command("ba表情")


@cmd_random_emoji.handle()
async def _(matcher: Matcher):
    try:
        emojis = await db_get_emoji()
        emo = await db_get(random.choice(emojis), True)
    except Exception:
        logger.exception("获取表情失败")
        await matcher.finish("获取表情失败，请检查后台输出")
    await matcher.finish(MessageSegment.image(emo))



import random
from typing import TYPE_CHECKING

from nonebot import on_command
from nonebot.adapters.onebot.v11 import Message, MessageSegment
from nonebot.internal.matcher import Matcher
from nonebot.log import logger
from nonebot.params import CommandArg

from ..config import config
from ..data.bawiki import recover_stu_alia, schale_to_gamekee
from ..data.gamekee import GameKeeVoice, game_kee_get_stu_li, game_kee_get_voice
from ..util import async_req

if TYPE_CHECKING:
    from . import HelpList

help_list: "HelpList" = [
    {
        "func": "学生语音",
        "trigger_method": "指令",
        "trigger_condition": "ba语音",
        "brief_des": "发送学生语音",
        "detail_des": (
            "从GameKee爬取学生语音并发送\n"
            "可以用 语音名称 或 中文或日文台词 搜索角色语音\n"
            "如果有多个结果，则会从中随机选择一个语音发送\n"
            " \n"
            "默认获取日配语音，如果角色有中配，则可以在指令后方带上 `中配` 来获取\n"
            "如果找不到中配语音对应的台词，则会展示日配台词\n"
            # " \n"
            # "可以用这些指令触发：\n"
            # "- <ft color=(238,120,0)>ba语音</ft>\n"
            " \n"
            "指令示例：\n"
            "- <ft color=(238,120,0)>ba语音 忧</ft>\n"
            "- <ft color=(238,120,0)>ba语音 美游 被cc</ft>\n"
            "- <ft color=(238,120,0)>ba语音 水大叔 好热</ft>\n"
            "- <ft color=(238,120,0)>ba语音中配 白子</ft>\n"
            "- <ft color=(238,120,0)>ba语音中配 大叔 睡午觉</ft>"
        ),
    },
]


cmd_voice = on_command("ba语音")


@cmd_voice.handle()
async def _(matcher: Matcher, cmd_arg: Message = CommandArg()):
    arg = cmd_arg.extract_plain_text().strip()
    is_chinese = False
    if arg.startswith("中配"):
        arg = arg[2:].strip()
        is_chinese = True

    if not arg:
        await matcher.finish("请提供学生名称")

    arg = arg.split()
    arg_len = len(arg)
    name = " ".join(arg[:-1]) if arg_len > 1 else arg[0]
    v_type = arg[-1].strip().lower() if arg_len > 1 else None

    try:
        ret = await game_kee_get_stu_li()
    except Exception:
        logger.exception("获取学生列表出错")
        await matcher.finish("获取学生列表出错，请检查后台输出")

    if not ret:
        await matcher.finish("没有获取到学生列表数据")

    try:
        org_stu_name = await recover_stu_alia(name, True)
        stu_name = await schale_to_gamekee(org_stu_name)
    except Exception:
        logger.exception("还原学生别名失败")
        await matcher.finish("还原学生别名失败，请检查后台输出")

    if not (stu_info := ret.get(stu_name)):
        await matcher.finish("未找到该学生")

    voices = await game_kee_get_voice(stu_info["content_id"], is_chinese)
    if v_type:
        voices = [
            x
            for x in voices
            if (
                (v_type in x.title.lower())
                or (v_type in x.jp.lower())
                or (v_type in x.cn.lower())
            )
        ]
    if not voices:
        await matcher.finish("没找到符合要求的语音捏")

    v: GameKeeVoice = random.choice(voices)
    voice_type = "中配" if is_chinese else "日配"
    im = [f"学生 {org_stu_name} {voice_type}语音 {v.title}"]
    if v.jp or v.cn:
        im.append("-=-=-=-=-=-=-=-")
        if v.jp:
            im.append(v.jp)
        if v.cn:
            im.append(v.cn)
    await matcher.send("\n".join(im))

    if config.ba_voice_use_card:
        await matcher.finish(
            MessageSegment(
                "music",
                {
                    "type": "custom",
                    "subtype": "163",
                    "url": v.url,
                    "voice": v.url,
                    "title": v.title,
                    "content": org_stu_name,
                    "image": f'http:{stu_info["icon"]}',
                },
            ),
        )

    v_data = await async_req(v.url, raw=True)
    await matcher.finish(MessageSegment.record(v_data))



from nonebot import on_command

from ..command import help_list

try:
    from .pic_menu import extra as extra
    from .pic_menu import help_handle
    from .pic_menu import usage as usage
except ImportError:
    from .manual import extra as extra
    from .manual import help_handle
    from .manual import usage as usage


def register_help_cmd():
    help_list.append(
        {
            "func": "插件帮助",
            "trigger_method": "指令",
            "trigger_condition": "ba帮助",
            "brief_des": "查看插件功能帮助",
            "detail_des": (
                "查看插件的功能列表，或某功能的详细介绍\n"
                "装载 PicMenu 插件后插件将会调用 PicMenu 生成帮助图片\n"
                " \n"
                "可以用这些指令触发：\n"
                "- <ft color=(238,120,0)>ba帮助</ft>\n"
                "- <ft color=(238,120,0)>ba菜单</ft>\n"
                "- <ft color=(238,120,0)>ba功能</ft>\n"
                " \n"
                "指令示例：\n"
                "- <ft color=(238,120,0)>ba帮助</ft>（功能列表）\n"
                "- <ft color=(238,120,0)>ba帮助 日程表</ft>（功能详情）"
            ),
        },
    )

    help_cmd = on_command("ba帮助", aliases={"ba菜单", "ba功能"})
    help_cmd.handle()(help_handle)



import asyncio
import contextlib
import itertools
import re
import time
from dataclasses import dataclass
from datetime import datetime
from io import BytesIO
from typing import Any, Dict, List, NoReturn, Union, cast

from bs4 import BeautifulSoup, ResultSet, Tag
from nonebot import logger
from nonebot.adapters.onebot.v11 import MessageSegment
from nonebot.internal.matcher import Matcher
from nonebot_plugin_htmlrender import get_new_page
from PIL.Image import Resampling
from pil_utils import BuildImage, text2image
from playwright.async_api import Page

from ..config import config
from ..resource import RES_CALENDER_BANNER, RES_GRADIENT_BG
from ..util import async_req, parse_time_delta


async def game_kee_request(url: str, **kwargs) -> Union[List, Dict[str, Any]]:
    ret = cast(
        dict,
        await async_req(
            f"{config.ba_gamekee_url}{url}",
            headers={"game-id": "0", "game-alias": "ba"},
            proxy=None,
            **kwargs,
        ),
    )
    if ret["code"] != 0:
        raise ConnectionError(ret["msg"])
    return ret["data"]


async def game_kee_get_calender() -> List[dict]:
    ret = cast(list, await game_kee_request("v1/wiki/index"))

    for i in ret:
        if i["module"]["id"] == 12:
            li: list = i["list"]

            now = time.time()
            li = [x for x in li if (now < x["end_at"])]

            li.sort(key=lambda x: x["begin_at"] if now < x["begin_at"] else x["end_at"])
            li.sort(key=lambda x: now < x["begin_at"])
            li.sort(key=lambda x: x["importance"], reverse=True)
            return li

    return []


async def game_kee_get_stu_li() -> Dict[str, dict]:
    ret = cast(dict, await game_kee_request("v1/wiki/entry"))

    for i in ret["entry_list"]:
        if i["id"] == 23941:
            for ii in i["child"]:
                if ii["id"] == 49443:
                    return {x["name"]: x for x in ii["child"]}

    return {}


async def game_kee_get_stu_cid_li() -> Dict[str, int]:
    return {x: y["content_id"] for x, y in (await game_kee_get_stu_li()).items()}


def game_kee_page_url(sid: int) -> str:
    return f"{config.ba_gamekee_url}{sid}.html"


async def game_kee_get_page(url: str) -> bytes:
    async with cast(Page, get_new_page()) as page:
        await page.goto(url, timeout=60 * 1000)

        # 删掉header
        await page.add_script_tag(
            content='document.getElementsByClassName("wiki-header")'
            ".forEach((v)=>{v.remove()})",
        )

        # 展开折叠的语音
        folds = await page.query_selector_all('xpath=//div[@class="fold-table-btn"]')
        for i in folds:
            with contextlib.suppress(Exception):
                await i.click()

        # 隐藏 header 和 footer
        js_str = "(obj) => { obj.style.display = 'none' }"
        await page.eval_on_selector(".wiki-header", js_str)
        await page.eval_on_selector(".wiki-footer", js_str)

        element = await page.query_selector('xpath=//div[@class="wiki-detail-body"]')
        if not element:
            raise ValueError
        return await element.screenshot()


async def send_wiki_page(sid: int, matcher: Matcher) -> NoReturn:
    url = game_kee_page_url(sid)
    await matcher.send(f"请稍等，正在截取Wiki页面……\n{url}")

    try:
        img = await game_kee_get_page(url)
    except Exception:
        logger.exception(f"截取wiki页面出错 {url}")
        await matcher.finish("截取页面出错，请检查后台输出")

    await matcher.finish(MessageSegment.image(img))


async def game_kee_calender() -> Union[MessageSegment, str]:
    ret = await game_kee_get_calender()
    if not ret:
        return "没有获取到GameKee日程表数据"

    pic = await game_kee_get_calender_page(ret)
    return MessageSegment.image(pic)


async def game_kee_get_calender_page(ret, has_pic=True) -> BytesIO:
    now = datetime.now()

    async def draw(it: dict):
        _p = None
        if has_pic and (_p := it.get("picture")):
            try:
                _p = (
                    BuildImage.open(BytesIO(await async_req(f"https:{_p}", raw=True)))
                    .resize_width(1290)
                    .circle_corner(15)
                )
            except Exception:
                logger.exception("下载日程表图片失败")

        begin = datetime.fromtimestamp(it["begin_at"])
        end = datetime.fromtimestamp(it["end_at"])
        started = begin <= now
        time_remain = (end if started else begin) - now
        dd, hh, mm, ss = parse_time_delta(time_remain)

        # logger.debug(f'{it["title"]} | {started} | {time_remain}')

        title_p = text2image(
            f'[b]{it["title"]}[/b]',
            "#ffffff00",
            max_width=1290,
            fontsize=65,
        )
        time_p = text2image(
            f"{begin} ~ {end}",
            "#ffffff00",
            max_width=1290,
            fontsize=40,
        )
        desc_p = (
            text2image(
                desc.replace("<br>", ""),
                "#ffffff00",
                max_width=1290,
                fontsize=40,
            )
            if (desc := it["description"])
            else None
        )
        remain_p = text2image(
            f"剩余 [color=#fc6475]{dd}[/color] 天 [color=#fc6475]{hh}[/color] 时 "
            f"[color=#fc6475]{mm}[/color] 分 [color=#fc6475]{ss}[/color] 秒"
            f'{"结束" if started else "开始"}',
            "#ffffff00",
            max_width=1290,
            fontsize=50,
        )

        h = (
            100
            + (title_p.height + 25)
            + (time_p.height + 25)
            + (_p.height + 25 if _p else 0)
            + (desc_p.height + 25 if desc_p else 0)
            + remain_p.height
        )
        img = BuildImage.new("RGBA", (1400, h), (255, 255, 255, 70)).draw_rectangle(
            (0, 0, 10, h),
            "#fc6475" if it["importance"] else "#4acf75",
        )

        if not started:
            img.draw_rectangle((1250, 0, 1400, 60), "gray")
            img.draw_text((1250, 0, 1400, 60), "未开始", max_fontsize=50, fill="white")

        ii = 50
        img.paste(title_p, (60, ii), True)
        ii += title_p.height + 25
        img.paste(time_p, (60, ii), True)
        ii += time_p.height + 25
        if _p:
            img.paste(_p, (60, ii), True)
            ii += _p.height + 25
        if desc_p:
            img.paste(desc_p, (60, ii), True)
            ii += desc_p.height + 25
        img.paste(remain_p, (60, ii), True)
        # img = img.circle_corner(15)

        return img

    pics: List[BuildImage] = await asyncio.gather(  # type: ignore
        *[draw(x) for x in ret],
    )

    bg_w = 1500
    bg_h = 200 + sum([x.height + 50 for x in pics])
    bg = (
        BuildImage.new("RGBA", (bg_w, bg_h))
        .paste(RES_CALENDER_BANNER.copy().resize((1500, 150)))
        .draw_text(
            (50, 0, 1480, 150),
            "GameKee丨活动日程",
            max_fontsize=100,
            weight="bold",
            fill="#ffffff",
            halign="left",
        )
        .paste(
            RES_GRADIENT_BG.copy().resize(
                (1500, bg_h - 150),
                resample=Resampling.NEAREST,
            ),
            (0, 150),
        )
    )

    index = 200
    for p in pics:
        bg.paste(p.circle_corner(10), (50, index), True)
        index += p.height + 50

    return bg.save_jpg()


async def game_kee_grab_l2d(cid) -> List[str]:
    ret = cast(
        dict,
        await game_kee_request(f"v1/content/detail/{cid}"),
    )
    content: str = ret["content"]

    x = content.find('<div class="input-wrapper">官方介绍</div>')
    x = content.find('class="slide-item" data-index="2"', x)
    y = content.find('data-index="3"', x)

    content: str = content[x:y]

    img = re.findall('data-real="([^"]*)"', content)

    return [f"https:{x}" for x in img]


@dataclass()
class GameKeeVoice:
    title: str
    jp: str
    cn: str
    url: str


def parse_voice_elem(elem: Tag) -> GameKeeVoice:
    url: str = cast(str, elem["src"])
    if not url.startswith("http"):
        url = f"https:{url}"

    tr1: Tag = elem.parent.parent.parent.parent  # type: ignore
    tds: ResultSet[Tag] = tr1.find_all("td")
    title = tds[0].text.strip()
    jp = "\n".join(tds[2].stripped_strings)

    tr2 = tr1.next_sibling
    cn = "\n".join(tr2.stripped_strings)  # type: ignore
    return GameKeeVoice(title, jp, cn, url)


def merge_voice_dialogue(voices: List[List[GameKeeVoice]]) -> List[List[GameKeeVoice]]:
    main_voices = voices[0]
    other_voice_entries = voices[1:]

    for voice_entry in other_voice_entries:
        for voice in (x for x in voice_entry if (not x.jp) or (not x.cn)):
            corresponding_voice = next(
                (x for x in main_voices if x.title == voice.title),
                None,
            )
            if not corresponding_voice:
                continue

            if not voice.jp:
                voice.jp = corresponding_voice.jp
            if not voice.cn:
                voice.cn = corresponding_voice.cn

    return voices


async def game_kee_get_voice(cid: int, is_chinese: bool = False) -> List[GameKeeVoice]:
    wiki_html = (
        cast(
            dict,
            await game_kee_request(f"v1/content/detail/{cid}"),
        )
    )["content"]
    bs = BeautifulSoup(wiki_html, "lxml")

    multi_lang_voices = [
        [parse_voice_elem(x) for x in audios]
        for table_fathers in bs.select(".slide-item")
        if (
            # 选择 tables
            (tables := table_fathers.select(".table-overflow > .mould-table"))
            # 检查 tables 中是否有语音，如果有，就取出语音到变量 aus -> audios
            and (
                audios := next(
                    (aus for child in tables if (aus := child.find_all("audio"))),
                    None,
                )
            )
        )
    ]
    if multi_lang_voices:
        return merge_voice_dialogue(multi_lang_voices)[1 if is_chinese else 0]

    # 没有中配
    if is_chinese:
        return []
    return list(
        itertools.chain(
            *(
                [parse_voice_elem(x) for x in a]
                for x in bs.select(".mould-table")
                if (a := x.find_all("audio"))
            )
        )
    )


async def get_level_list() -> Dict[str, int]:
    entry = cast(dict, await game_kee_request("v1/wiki/entry"))
    entry_list: List[Dict] = entry["entry_list"]
    guide_entry: List[Dict] = next(x["child"] for x in entry_list if x["id"] == 50284)
    levels = itertools.chain(
        *(x["child"] for x in guide_entry if cast(str, x["name"]).endswith("章")),
    )
    return {
        n.upper(): x["content_id"]
        for x in levels
        if re.match(r"^(H)?(TR|\d+)-(\d+)$", (n := cast(str, x["name"])))
    }


async def extract_content_pic(cid: int) -> List[str]:
    wiki_html = (
        cast(
            dict,
            await game_kee_request(f"v1/content/detail/{cid}"),
        )
    )["content"]
    bs = BeautifulSoup(wiki_html, "lxml")
    img_elem = bs.find_all("img")
    img_urls = cast(List[str], [x["src"] for x in img_elem])
    img_urls = [f"https:{x}" if x.startswith("//") else x for x in img_urls]
    return img_urls



import datetime
import json
from dataclasses import dataclass
from typing import (
    Any,
    Dict,
    Iterable,
    List,
    Literal,
    Optional,
    TypeVar,
    Union,
    cast,
    overload,
)

from httpx import AsyncClient
from nonebot.adapters.onebot.v11 import Message
from nonebot_plugin_apscheduler import scheduler
from PIL import Image, ImageOps

from .config import config

T = TypeVar("T")
NestedIterable = Iterable[Union[T, Iterable["NestedIterable[T]"]]]

req_cache: Dict[str, "RequestCache"] = {}


@dataclass
class RequestCache:
    method: str
    raw: bool
    json: bool
    params: str
    content: Any


def get_req_cache(
    url: str,
    method: Optional[str] = None,
    raw: Optional[bool] = None,
    json: Optional[bool] = None,
    params: Optional[str] = None,
) -> Optional[Any]:
    if (c := req_cache.get(url)) and (
        (method is None or method == c.method)
        and (raw is None or raw == c.raw)
        and (json is None or json == c.json)
        and (params is None or params == c.params)
    ):
        return c.content
    return None


def clear_req_cache():
    req_cache.clear()


if config.ba_clear_req_cache_interval > 0:

    @scheduler.scheduled_job("interval", hours=config.ba_clear_req_cache_interval)
    async def _():
        clear_req_cache()


@overload
async def async_req(
    url,
    *,
    is_json: Literal[True] = True,
    raw: Literal[False] = False,
    ignore_cache: bool = False,
    proxy: Optional[str] = config.ba_proxy,
    method: str = "GET",
    params: Optional[Dict[str, Any]] = None,
    **kwargs,
) -> Any:
    ...


@overload
async def async_req(
    url,
    *,
    is_json: Literal[False] = False,
    raw: Literal[True] = True,
    ignore_cache: bool = False,
    proxy: Optional[str] = config.ba_proxy,
    method: str = "GET",
    params: Optional[Dict[str, Any]] = None,
    **kwargs,
) -> bytes:
    ...


@overload
async def async_req(
    url,
    *,
    is_json: Literal[False] = False,
    raw: Literal[False] = False,
    ignore_cache: bool = False,
    proxy: Optional[str] = config.ba_proxy,
    method: str = "GET",
    params: Optional[Dict[str, Any]] = None,
    **kwargs,
) -> str:
    ...


async def async_req(
    url,
    is_json=True,
    raw=False,
    ignore_cache=False,
    proxy=config.ba_proxy,
    method="GET",
    params=None,
    **kwargs,
):
    param_json = json.dumps(params) if params else "{}"

    if (not ignore_cache) and (
        c := get_req_cache(url, method, raw, is_json, param_json)
    ):
        return c

    async with AsyncClient(proxies=proxy) as cli:
        resp = await cli.request(method, url, params=params, **kwargs)
        resp.raise_for_status()

        if (not raw) and is_json:
            ret = resp.json()
        elif raw:
            ret = resp.content
        else:
            ret = resp.text

        req_cache[url] = RequestCache(method, raw, is_json, param_json, ret)
        return ret


def format_timestamp(t: int):
    return datetime.datetime.fromtimestamp(t).strftime("%Y-%m-%d %H:%M:%S")


def recover_alia(origin: str, alia_dict: Dict[str, List[str]]):
    origin = replace_brackets(origin).strip()
    origin_ = origin.lower()

    # 精确匹配
    for k, li in alia_dict.items():
        if origin_ in li or origin_ == k:
            return k

    # 没找到，模糊匹配
    origin_ = origin.replace(" ", "")
    for k, li in alia_dict.items():
        li = [x.replace(" ", "") for x in ([k, *li])]
        for v in li:
            if origin_ in v:
                return k

    return origin


def parse_time_delta(t: datetime.timedelta):
    mm, ss = divmod(t.seconds, 60)
    hh, mm = divmod(mm, 60)
    dd = t.days or 0
    return dd, hh, mm, ss


def img_invert_rgba(im: Image.Image):
    # https://stackoverflow.com/questions/2498875/how-to-invert-colors-of-image-with-pil-python-imaging
    r, g, b, a = im.split()
    rgb_image = Image.merge("RGB", (r, g, b))
    inverted_image = ImageOps.invert(rgb_image)
    r2, g2, b2 = inverted_image.split()
    return Image.merge("RGBA", (r2, g2, b2, a))


def replace_brackets(original: str):
    return original.replace("（", "(").replace("）", "(")


def splice_msg(msgs: list) -> Message:
    im = Message()
    for i, v in enumerate(msgs):
        if isinstance(v, str) and (i != 0):
            v = f"\n{v}"
        im += v
    return im


def split_list(li: Iterable[T], length: int) -> List[List[T]]:
    latest = []
    tmp = []
    for n, i in enumerate(li):
        tmp.append(i)
        if (n + 1) % length == 0:
            latest.append(tmp)
            tmp = []
    if tmp:
        latest.append(tmp)
    return latest



from typing import TYPE_CHECKING, List, NoReturn

from nonebot import logger, on_command
from nonebot.adapters.onebot.v11 import Message, MessageEvent, MessageSegment
from nonebot.internal.matcher import Matcher
from nonebot.params import ArgPlainText, CommandArg
from nonebot.typing import T_State

from ..data.arona import IMAGE_TYPE_MAP, ImageModel, get_image, search

if TYPE_CHECKING:
    from . import HelpList

help_list: "HelpList" = [
    {
        "func": "Arona数据源攻略",
        "trigger_method": "指令",
        "trigger_condition": "arona",
        "brief_des": "从Arona Bot数据源中搜索攻略图",
        "detail_des": (
            "从Arona Bot数据源中搜索攻略图，支持模糊搜索\n"
            " \n"
            "可以搜索的内容：\n"
            "- <ft color=(238,120,0)>学生攻略图</ft>（星野，白子 等）\n"
            "- <ft color=(238,120,0)>主线地图</ft>（1-1，H1-1 等）\n"
            "- <ft color=(238,120,0)>杂图</ft>（使用 <ft color=(238,120,0)>arona 杂图</ft> 可以获取图片列表）\n"
            " \n"
            "可以用这些指令触发：\n"
            "- <ft color=(238,120,0)>arona</ft>\n"
            "- <ft color=(238,120,0)>Arona</ft>\n"
            "- <ft color=(238,120,0)>ARONA</ft>\n"
            "- <ft color=(238,120,0)>阿罗娜</ft>\n"
            " \n"
            "指令示例：\n"
            "- <ft color=(238,120,0)>arona</ft>（会向你提问需要搜索什么）\n"
            "- <ft color=(238,120,0)>arona 国际服未来视</ft>（精确搜索）\n"
            "- <ft color=(238,120,0)>arona 国际服</ft>（模糊搜索）"
        ),
    },
]


cmd_arona = on_command("arona", aliases={"Arona", "ARONA", "阿罗娜"})


async def send_image(matcher: Matcher, img: ImageModel) -> NoReturn:
    try:
        res = await get_image(img.path, img.hash)
    except Exception:
        logger.exception("Arona数据源图片获取失败")
        await matcher.finish("图片获取失败，请稍后再试")

    await matcher.finish(MessageSegment.image(res))


@cmd_arona.handle()
async def _(matcher: Matcher, arg: Message = CommandArg()):
    if arg.extract_plain_text().strip():
        matcher.set_arg("param", arg)


@cmd_arona.got("param", prompt="请输入要搜索的关键词")
async def _(matcher: Matcher, state: T_State, param: str = ArgPlainText()):
    param = param.strip()
    if not param:
        await matcher.reject("关键词不能为空，请重新输入")

    try:
        res = await search(param)
    except Exception:
        logger.exception("Arona数据源搜索失败")
        await matcher.finish("搜索失败，请稍后再试")

    if not res:
        await matcher.finish("没有找到相关图片")

    if len(res) == 1:
        await send_image(matcher, res[0])

    state["res"] = res
    list_txt = "\n".join(f"{i}. {r.name}" for i, r in enumerate(res, 1))
    await matcher.pause(
        f"Arona 查到了多个结果，请 sensei 发送序号选择~\n{list_txt}\nTip：发送 0 取消选择",
    )


@cmd_arona.handle()
async def _(event: MessageEvent, matcher: Matcher, state: T_State):
    index_str = event.get_plaintext().strip()
    res: List[ImageModel] = state["res"]

    if not (index_str.isdigit() and (0 <= (index := int(index_str)) <= len(res))):
        await matcher.reject("序号错误，请重新输入")

    if index == 0:
        await matcher.finish("操作已取消")

    param = res[index - 1].name
    try:
        final_res = await search(param)
        assert final_res
    except Exception:
        logger.exception("Arona数据源搜索失败")
        await matcher.finish("搜索失败，请稍后再试")

    await send_image(matcher, final_res[0])



from typing import TYPE_CHECKING

from nonebot import on_command
from nonebot.adapters.onebot.v11 import Message
from nonebot.internal.matcher import Matcher
from nonebot.log import logger
from nonebot.params import CommandArg

from ..data.bawiki import recover_stu_alia
from ..data.gamekee import game_kee_get_stu_cid_li, send_wiki_page

if TYPE_CHECKING:
    from . import HelpList

help_list: "HelpList" = [
    {
        "func": "学生Wiki",
        "trigger_method": "指令",
        "trigger_condition": "ba学生wiki",
        "brief_des": "查询学生详情（GameKee）",
        "detail_des": (
            "访问对应学生GameKee Wiki页面并截图，支持部分学生别名\n"
            " \n"
            "指令示例：\n"
            "- <ft color=(238,120,0)>ba学生wiki 白子</ft>\n"
            "- <ft color=(238,120,0)>ba学生wiki xcw</ft>"
        ),
    },
]


cmd_stu_wiki = on_command("ba学生wiki", aliases={"ba学生Wiki", "ba学生WIKI"})


@cmd_stu_wiki.handle()
async def _(matcher: Matcher, cmd_arg: Message = CommandArg()):
    arg = cmd_arg.extract_plain_text().strip()
    if not arg:
        await matcher.finish("请提供学生名称")

    try:
        ret = await game_kee_get_stu_cid_li()
    except Exception:
        logger.exception("获取学生列表出错")
        await matcher.finish("获取学生列表出错，请检查后台输出")

    if not ret:
        await matcher.finish("没有获取到学生列表数据")

    if not (sid := ret.get(await recover_stu_alia(arg, True))):
        await matcher.finish("未找到该学生")

    await send_wiki_page(sid, matcher)



import hashlib
import shutil
from typing import List, Optional

import anyio
from pydantic import BaseModel

from ..config import config
from ..resource import CACHE_PATH
from ..util import async_req

ARONA_CACHE_PATH = CACHE_PATH / "arona"
if ARONA_CACHE_PATH.exists() and config.ba_auto_clear_arona_cache:
    shutil.rmtree(ARONA_CACHE_PATH)
if not ARONA_CACHE_PATH.exists():
    ARONA_CACHE_PATH.mkdir(parents=True)

IMAGE_TYPE_MAP = {
    1: "学生攻略",
    2: "主线地图",
    3: "杂图",
}


class ImageModel(BaseModel):
    name: str
    path: str
    hash: str  # noqa: A003
    type: int  # noqa: A003


class ImageAPIResult(BaseModel):
    status: int
    data: Optional[List[ImageModel]]
    message: str


async def get_image(path: str, hash_str: Optional[str] = None) -> bytes:
    if not path.startswith("/"):
        raise ValueError("path must start with /")

    if hash_str:
        file_path = anyio.Path(ARONA_CACHE_PATH / hash_str)
        if await file_path.exists():
            return await file_path.read_bytes()

    content = await async_req(f"{config.ba_arona_cdn_url}image{path}", raw=True)

    if not hash_str:
        hash_str = hashlib.md5(content).hexdigest()

    file_path = anyio.Path(ARONA_CACHE_PATH / hash_str)
    await file_path.write_bytes(content)

    return content


async def search(name: str) -> Optional[List[ImageModel]]:
    resp: dict = await async_req(
        f"{config.ba_arona_api_url}api/v1/image",
        params={"name": name},
    )
    return ImageAPIResult(**resp).data



from typing import TYPE_CHECKING

from nonebot import on_command
from nonebot.adapters.onebot.v11 import Message
from nonebot.internal.matcher import Matcher
from nonebot.log import logger

from ..data.bawiki import db_wiki_furniture

if TYPE_CHECKING:
    from . import HelpList

help_list: "HelpList" = [
    {
        "func": "互动家具总览",
        "trigger_method": "指令",
        "trigger_condition": "ba互动家具",
        "brief_des": "查询互动家具总览图",
        "detail_des": "发送咖啡厅内所有互动家具以及对应学生的总览图\n图片作者 B站@夜猫咪喵喵猫",
    },
]


cmd_furniture_wiki = on_command("ba互动家具")


@cmd_furniture_wiki.handle()
async def _(matcher: Matcher):
    try:
        im = await db_wiki_furniture()
    except Exception:
        logger.exception("获取互动家具wiki图片错误")
        await matcher.finish("获取图片失败，请检查后台输出")

    await matcher.finish(Message(im))



import asyncio
import json
import random
import time
from dataclasses import dataclass
from io import BytesIO
from typing import Dict, Iterable, List, Optional, TypedDict, Union, cast

import aiofiles
from nonebot import logger
from nonebot.adapters.onebot.v11 import MessageSegment
from pil_utils import BuildImage

from ..config import config
from ..resource import (
    DATA_PATH,
    RES_GACHA_BG,
    RES_GACHA_CARD_BG,
    RES_GACHA_CARD_MASK,
    RES_GACHA_NEW,
    RES_GACHA_PICKUP,
    RES_GACHA_STAR,
    RES_GACHA_STU_ERR,
)
from ..util import split_list
from .schaledb import schale_get, schale_get_stu_dict

GACHA_DATA_PATH = DATA_PATH / "gacha.json"
if not GACHA_DATA_PATH.exists():
    GACHA_DATA_PATH.write_text("{}")


DEFAULT_GACHA_DATA: "GachaData" = {"collected": [], "total_count": 0}
COOL_DOWN_DICT: Dict[str, float] = {}


class GachaData(TypedDict):
    collected: List[int]
    total_count: int


@dataclass()
class GachaStudent:
    id: int  # noqa: A003
    new: bool = False
    pickup: bool = False


def get_gacha_cool_down(
    user: Union[str, int],
    group: Optional[Union[str, int]] = None,
) -> int:
    key = f"{group}.{user}" if group else f"{user}"
    now = time.time()

    if last := COOL_DOWN_DICT.get(key):
        remain = config.ba_gacha_cool_down - round(now - last)
        return remain if remain >= 0 else 0

    return 0


def set_gacha_cool_down(user: Union[str, int], group: Optional[Union[str, int]] = None):
    key = f"{group}.{user}" if group else f"{user}"
    COOL_DOWN_DICT[key] = time.time()


async def set_gacha_data(qq: str, data: GachaData):
    async with aiofiles.open(str(GACHA_DATA_PATH), "r+", encoding="u8") as f:
        j = json.loads(await f.read())
        j[qq] = data

        await f.seek(0)
        await f.truncate()

        await f.write(json.dumps(j))


async def get_gacha_data(qq: str) -> GachaData:
    async with aiofiles.open(str(GACHA_DATA_PATH), encoding="u8") as f:
        j = await f.read()

    data: Dict[str, GachaData] = json.loads(j)
    if not (user_data := data.get(qq)):
        user_data = DEFAULT_GACHA_DATA.copy()
        await set_gacha_data(qq, user_data)

    return user_data


async def gen_stu_img(students: Iterable[GachaStudent]) -> List[BuildImage]:
    stu_li = await schale_get_stu_dict("Id")

    async def gen_single(stu: GachaStudent) -> BuildImage:
        bg = RES_GACHA_CARD_BG.copy()

        stu_star = 0
        try:
            stu_j = stu_li[stu.id]
            stu_star: int = stu_j["StarGrade"]
            stu_img = await schale_get(
                f"images/student/collection/{stu_j['CollectionTexture']}.webp",
                True,
            )
            stu_img = BuildImage.open(BytesIO(stu_img))
        except Exception:
            logger.exception(f"学生数据获取失败 {stu.id}")
            stu_img = RES_GACHA_STU_ERR

        card_img = BuildImage.new("RGBA", RES_GACHA_CARD_MASK.size, (0, 0, 0, 0))
        card_img.image.paste(
            stu_img.resize(RES_GACHA_CARD_MASK.size, keep_ratio=True).image,
            mask=RES_GACHA_CARD_MASK.image,
        )

        bg = bg.paste(card_img, (26, 13), True)

        star_x_offset = int(26 + (159 - 30 * stu_star) / 2)
        star_y_offset = 198
        for i in range(stu_star):
            bg = bg.paste(RES_GACHA_STAR, (star_x_offset + i * 30, star_y_offset), True)

        font_x_offset = 45
        font_y_offset = 2

        if stu.new:
            bg = bg.paste(RES_GACHA_NEW, (font_x_offset, font_y_offset), True)
            font_x_offset -= 2
            font_y_offset += 29

        if stu.pickup:
            font_x_offset -= 4
            font_y_offset -= 4
            bg = bg.paste(RES_GACHA_PICKUP, (font_x_offset, font_y_offset), True)

        return bg

    return await asyncio.gather(*[gen_single(x) for x in students])


async def gen_gacha_img(students: Iterable[GachaStudent], count: int) -> BytesIO:
    line_limit = 5
    stu_cards = split_list(await gen_stu_img(students), line_limit)
    card_w, card_h = stu_cards[0][0].size

    bg = RES_GACHA_BG.copy()

    x_gap = 10
    y_gap = 80
    y_offset = int((bg.height - (len(stu_cards) * (y_gap + card_h) - y_gap)) / 2)
    for line in stu_cards:
        x_offset = int((bg.width - (len(line) * (x_gap + card_w) - x_gap)) / 2)
        for card in line:
            bg = bg.paste(card, (x_offset, y_offset), True)
            x_offset += card_w + x_gap
        y_offset += card_h + y_gap

    bg = bg.draw_text(
        (1678, 841, 1888, 885),
        "距上个3★UP",
        max_fontsize=30,
        weight="bold",
        fill=(36, 90, 126),
    ).draw_text(
        (1643, 885, 1890, 935),
        str(count),
        max_fontsize=30,
        weight="bold",
        fill=(255, 255, 255),
    )

    return bg.save("PNG")


async def gacha(
    qq: str,
    times: int,
    gacha_data_json: dict,
    up_pool: Optional[List[int]] = None,
):
    # 屎山代码 别骂了别骂了
    # 如果有大佬指点指点怎么优化或者愿意发个PR就真的太感激了

    if not up_pool:
        up_pool = []

    stu_li = await schale_get_stu_dict("Id")
    up_3_li, up_2_li = [
        [x for x in up_pool if x in stu_li and stu_li[x]["StarGrade"] == y]
        for y in [3, 2]
    ]

    base_char: dict = gacha_data_json["base"]
    for up in up_pool:
        for li in cast(List[List[int]], base_char.values()):
            if up in li:
                li.remove(up)

    star_3_base, star_2_base, star_1_base = [base_char[x] for x in ["3", "2", "1"]]
    star_3_chance, star_2_chance, star_1_chance = [
        x["chance"] for x in [star_3_base, star_2_base, star_1_base]
    ]

    up_3_chance = 0
    up_2_chance = 0
    if up_3_li:
        up_3_chance = gacha_data_json["up"]["3"]["chance"]
        star_3_chance -= up_3_chance
    if up_2_li:
        up_2_chance = gacha_data_json["up"]["2"]["chance"]
        star_2_chance -= up_2_chance

    gacha_data = await get_gacha_data(qq)
    gacha_result: List[GachaStudent] = []

    picked_3star_up = False
    last_count = gacha_data["total_count"]
    for i in range(1, times + 1):
        pool_and_weight = [
            (up_3_li, up_3_chance),
            (up_2_li, up_2_chance),
            (star_3_base["char"], star_3_chance),
            (star_2_base["char"], star_2_chance),
        ]
        if i % 10 != 0:
            pool_and_weight.append((star_1_base["char"], star_1_chance))

        pool_and_weight = [x for x in pool_and_weight if x[0]]
        pool = [x[0] for x in pool_and_weight]
        weight = [x[1] for x in pool_and_weight]

        random.seed()
        char = random.choice(random.choices(pool, weights=weight, k=1)[0])

        is_3star_pickup = char in up_3_li
        is_pickup = is_3star_pickup or (char in up_2_li)
        is_new = char not in gacha_data["collected"]
        gacha_result.append(GachaStudent(id=char, pickup=is_pickup, new=is_new))

        if is_new:
            gacha_data["collected"].append(char)

        if is_3star_pickup or ((not up_pool) and char in star_3_base["char"]):
            gacha_data["total_count"] = 0
            picked_3star_up = True
        else:
            gacha_data["total_count"] += 1
            if not picked_3star_up:
                last_count += 1

    await set_gacha_data(qq, gacha_data)
    return MessageSegment.image(await gen_gacha_img(gacha_result, last_count))
